@@ -1,37 +1,43 @@
 #' Convert a phyloseq to taxmap
-#' 
+#'
 #' Converts a phyloseq object to a taxmap object.
-#' 
+#'
 #' @param obj A phyloseq object
-#' 
+#' @param class_regex A regular expression used to parse data in the taxon
+#'   names. There must be a capture group (a pair of parentheses) for each item
+#'   in \code{class_key}. See \code{\link[taxa]{parse_tax_data}} for examples of
+#'   how this works.
+#' @inheritParams taxa::parse_tax_data
+#'
 #' @return A taxmap object
-#' 
+#'
 #' @family parsers
-#' 
+#'
 #' @examples \dontrun{
-#' 
+#'
 #' # Install phyloseq to get example data
 #' # source('http://bioconductor.org/biocLite.R')
 #' # biocLite('phyloseq')
-#' 
+#'
 #' # Parse example dataset
 #' library(phyloseq)
 #' data(GlobalPatterns)
 #' x <- parse_phyloseq(GlobalPatterns)
-#' 
+#'
 #' # Plot data
 #' heat_tree(x,
 #'           node_size = n_obs,
 #'           node_color = n_obs,
 #'           node_label = taxon_names,
 #'           tree_label = taxon_names)
-#' 
+#'
 #' }
-#' 
+#'
 #' @import taxa
-#' 
+#'
 #' @export
-parse_phyloseq <- function(obj) {
+parse_phyloseq <- function(obj, class_regex = "(.*)",
+                           class_key = "taxon_name") {
   datasets <- list()
   mappings <- c()
   
@@ -76,12 +82,14 @@ parse_phyloseq <- function(obj) {
   }
   
   # Construct output
-  tax_cols <- colnames(tax_data)[which(tolower(colnames(tax_data)) %in% possible_ranks)]
+  tax_cols <- colnames(tax_data)
   output <- taxa::parse_tax_data(tax_data = tax_data, 
                                  datasets = datasets,
                                  class_cols = tax_cols, 
                                  mappings = mappings,
-                                 named_by_rank = TRUE)
+                                 named_by_rank = TRUE,
+                                 class_regex = class_regex,
+                                 class_key = class_key)
   
   # Remove NA taxa
   output$filter_taxa(output$taxon_names() != "NA")
@@ -149,16 +157,16 @@ parse_phyloseq <- function(obj) {
 parse_mothur_tax_summary <- function(file = NULL, text = NULL, table = NULL) {
   
   # Check that `file` and `text` and `table` are not used together
-  are_missing <- c(file = missing(file),
-                   text = missing(text),
-                   table = missing(table))
+  are_missing <- c(file = is.null(file),
+                   text = is.null(text),
+                   table = is.null(table))
   if (sum(are_missing) != 2) {
     stop(paste0('Either "file", "text", or "table" must be supplied, but only one.'))
   }
   
   # Read raw data
   if (! are_missing["file"]) {
-    raw_data <- utils::read.csv(file_path = file, header = TRUE, sep = "\t",
+    raw_data <- utils::read.csv(file = file, header = TRUE, sep = "\t",
                                 stringsAsFactors = FALSE)
   } else if (! are_missing["text"]) {
     raw_data <- utils::read.csv(text = text, header = TRUE, sep = "\t",
@@ -290,12 +298,18 @@ parse_mothur_taxonomy <- function(file = NULL, text = NULL) {
 #' Parse a BIOM output from QIIME
 #' 
 #' Parses a file in BIOM format from QIIME into a taxmap object.
+#' This also seems to work with files from MEGAN.
 #' I have not tested if it works with other BIOM files. 
 #' 
 #' This function was inspired by the tutorial created by Geoffrey Zahn at 
 #' http://geoffreyzahn.com/getting-your-otu-table-into-r/.
 #' 
 #' @param file (\code{character} of length 1) The file path to the input file.
+#' @param class_regex A regular expression used to parse data in the taxon
+#'   names. There must be a capture group (a pair of parentheses) for each item
+#'   in \code{class_key}. See \code{\link[taxa]{parse_tax_data}} for examples of
+#'   how this works.
+#' @inheritParams taxa::parse_tax_data
 #' 
 #' @return A taxmap object
 #' 
@@ -304,30 +318,57 @@ parse_mothur_taxonomy <- function(file = NULL, text = NULL) {
 #' @import taxa
 #' 
 #' @export
-parse_qiime_biom <- function(file) {
+parse_qiime_biom <- function(file, class_regex = "(.*)",
+                             class_key = "taxon_name") {
   # Check that the "biomformat" package has been installed
   check_for_pkg("biomformat")
   
   # Read biom file
   my_biom <- biomformat::read_biom(file)
   
-  # Coerce into a matrix
-  otu_table <- as.data.frame(as.matrix(biomformat::biom_data(my_biom)))
-  otu_table <- cbind(list(otu_id = rownames(taxonomy)), otu_table)
-  
   # Get taxonomy
   taxonomy <- biomformat::observation_metadata(my_biom)
+  if (is.null(taxonomy)) {
+    stop(call. = FALSE,
+         'Could not find taxonomy data in "', file,
+         '". If it does have taxonomy data, then it is not where this function expects it to be. ',
+         'If you think this is a bug, let us know at "', repo_url(), '/issues"')
+  }
   tax_cols <- colnames(taxonomy)
   
+  # Get OTU IDs
+  if (is.data.frame(taxonomy)) {
+    otu_ids <- rownames(taxonomy)
+  } else {
+    otu_ids <- names(taxonomy)
+  }
+  
+  # Coerce into a matrix
+  otu_table <- as.data.frame(as.matrix(biomformat::biom_data(my_biom)),
+                             stringsAsFactors = FALSE)
+  otu_table <- cbind(list(otu_id = otu_ids), otu_table,
+                     stringsAsFactors = FALSE)
+  
   # Get sample metadata (not used yet)
   metadata <- biomformat::sample_metadata(my_biom)
   
   # Create a taxmap object
-  output <- taxa::parse_tax_data(tax_data = taxonomy,
-                                 class_cols = colnames(taxonomy),
-                                 datasets = list(otu_table = otu_table),
-                                 mappings = c("{{name}}" = "{{name}}"),
-                                 include_tax_data = FALSE)
+  if (is.data.frame(taxonomy)) {
+    output <- taxa::parse_tax_data(tax_data = taxonomy,
+                                   class_cols = colnames(taxonomy),
+                                   datasets = list(otu_table = otu_table),
+                                   mappings = c("{{name}}" = "{{name}}"),
+                                   include_tax_data = FALSE,
+                                   class_regex = class_regex,
+                                   class_key = class_key)
+  } else {
+    output <- taxa::parse_tax_data(tax_data = taxonomy,
+                                   datasets = list(otu_table = otu_table),
+                                   mappings = c("{{name}}" = "{{name}}"),
+                                   include_tax_data = FALSE,
+                                   class_regex = class_regex,
+                                   class_key = class_key)
+  }
   
   return(output)
 }
@@ -344,7 +385,8 @@ parse_qiime_biom <- function(file) {
 #' (dog:20, (elephant:30, horse:60):20):50;
 #' }
 #' 
-#' @param file (\code{character} of length 1) The file path to the input file.
+#' @param file (\code{character} of length 1) The file path to the input file. Either \code{file} or \code{text} must be supplied but not both.
+#' @param text (\code{character} of length 1) The raw text to parse. Either \code{file} or \code{text} must be supplied but not both.
 #' 
 #' @return \code{\link{taxmap}}
 #' 
@@ -353,7 +395,18 @@ parse_qiime_biom <- function(file) {
 #' @import taxa
 #' 
 #' @export
-parse_newick <- function(file) {
+parse_newick <- function(file = NULL, text = NULL) {
+  # Check that `file` and `text` and `table` are not used together
+ if (sum(c(is.null(file), is.null(text))) != 1) {
+    stop(paste0('Either "file" or "text" must be supplied, but not both.'))
+  }
+  
+  # Read raw data
+  if (is.null(file)) {
+    file <- tempfile()
+    readr::write_lines(text, file)
+  }
+  
   # Read input
   raw_data <- phylotate::read_annotated(file, format = "newick")
   
@@ -520,16 +573,17 @@ parse_silva_fasta <- function(file = NULL, input = NULL, include_seqs = TRUE) {
   raw_headers <- names(raw_data)
   
   # Make classifications easier to parse
-  name_chars <- "A-Za-z0-9.\\-_+ "
+  name_chars <- "A-Za-z0-9._+ ='\"\\-"
   parts <- stringr::str_match(raw_headers,
                               paste0("^(.+;)([", name_chars, "]+)(\\(?.*\\)?)$"))
   parts <- as.data.frame(parts[, -1], stringsAsFactors = FALSE)
   colnames(parts) <- c("tax", "binom", "common")
   parts$binom <- sub(parts$binom, pattern = "sp\\. ", replacement = "sp\\._")
   parts$binom <- sub(parts$binom, pattern = "uncultured ", replacement = "uncultured_")
-  parts$binom <- gsub(pattern = " ", replacement = ";", parts$binom) 
+  # parts$binom <- gsub(pattern = " ", replacement = ";", parts$binom) 
   parts$binom <- sub(pattern = ";$", replacement = " ", parts$binom)
   headers <- apply(parts, MARGIN = 1, paste0, collapse = "")
+  headers <- gsub(headers, pattern = "\\[|\\]", replacement = "")
   
   # Create taxmap object
   output <- taxa::extract_tax_data(tax_data = headers,
